@@ -32,6 +32,7 @@ module Spy
     def initialize(object, method_name, singleton_method = true)
       @base_object, @method_name = object, method_name
       @singleton_method = singleton_method
+      @plan = nil
       reset!
     end
 
@@ -51,8 +52,21 @@ module Spy
         @original_method = current_method
       end
 
-      define_method_with = singleton_method ? :define_singleton_method : :define_method
-      base_object.send(define_method_with, method_name, override_method)
+      if original_method && original_method.owner == base_object
+        original_method.owner.send(:remove_method, method_name)
+      end
+
+      if singleton_method
+        if base_object.singleton_class.method_defined?(method_name) || base_object.singleton_class.private_method_defined?(method_name)
+          base_object.singleton_class.alias_method(method_name, method_name)
+        end
+        base_object.define_singleton_method(method_name, override_method)
+      else
+        if base_object.method_defined?(method_name) || base_object.private_method_defined?(method_name)
+          base_object.alias_method(method_name, method_name)
+        end
+        base_object.define_method(method_name, override_method)
+      end
 
       if [:public, :protected, :private].include? hook_opts[:visibility]
         method_owner.send(hook_opts[:visibility], method_name)
@@ -68,11 +82,10 @@ module Spy
     def unhook
       raise NeverHookedError, "'#{method_name}' method has not been hooked" unless hooked?
 
+      method_owner.send(:remove_method, method_name)
       if original_method && method_owner == original_method.owner
-        method_owner.send(:define_method, method_name, original_method)
-        method_owner.send(original_method_visibility, method_name) if original_method_visibility
-      else
-        method_owner.send(:remove_method, method_name)
+        original_method.owner.send(:define_method, method_name, original_method)
+        original_method.owner.send(original_method_visibility, method_name) if original_method_visibility
       end
 
       clear_method!
@@ -104,7 +117,7 @@ module Spy
     #   spy.and_return(force: true) { |invalid_arity| true }
     #
     # @return [self]
-    def and_return(value = nil)
+    def and_return(value = nil, &block)
       @do_not_check_plan_arity = false
 
       if block_given?
@@ -114,7 +127,7 @@ module Spy
           raise ArgumentError, "value and block conflict. Choose one"
         end
 
-        @plan = Proc.new
+        @plan = block
         check_for_too_many_arguments!(@plan)
       else
         @plan = Proc.new { value }
@@ -135,13 +148,29 @@ module Spy
     # tells the spy to call the original method
     # @return [self]
     def and_call_through
-      @plan = Proc.new do |*args, &block|
-        if original_method
-          original_method.call(*args, &block)
-        else
-          base_object.send(:method_missing, method_name, *args, &block)
+      if @base_object.is_a? Class
+        @plan = Proc.new do |object, *args, &block|
+          if original_method
+            if original_method.is_a? UnboundMethod
+              bound_method = original_method.bind(object)
+              bound_method.call(*args, &block)
+            else
+              original_method.call(*args, &block)
+            end
+          else
+            base_object.send(:method_missing, method_name, *args, &block)
+          end
+        end
+      else
+        @plan = Proc.new do |*args, &block|
+          if original_method
+            original_method.call(*args, &block)
+          else
+            base_object.send(:method_missing, method_name, *args, &block)
+          end
         end
       end
+
       self
     end
 
@@ -167,6 +196,7 @@ module Spy
       end
 
       @plan = Proc.new { raise exception }
+      self
     end
 
     # @overload and_throw(symbol)
@@ -191,9 +221,9 @@ module Spy
     # check if the method was called with the exact arguments
     # @param args Arguments that should have been sent to the method
     # @return [Boolean]
-    def has_been_called_with?(*args)
+    def has_been_called_with?(*args, &block)
       raise NeverHookedError unless @was_hooked
-      match = block_given? ? Proc.new : proc { |call| call.args == args }
+      match = block_given? ? block : proc { |call| call.args == args }
       calls.any?(&match)
     end
 
@@ -201,10 +231,18 @@ module Spy
     # method.
     def invoke(object, args, block, called_from)
       check_arity!(args.size)
-      result = if @plan
-                 check_for_too_many_arguments!(@plan)
-                 @plan.call(*args, &block)
-               end
+
+      if base_object.is_a? Class
+        result = if @plan
+                   check_for_too_many_arguments!(@plan)
+                   @plan.call(object, *args, &block)
+                 end
+      else
+        result = if @plan
+                   check_for_too_many_arguments!(@plan)
+                   @plan.call(*args, &block)
+                 end
+      end
     ensure
       calls << CallLog.new(object, called_from, args, block, result)
     end
@@ -272,6 +310,8 @@ module Spy
       min_arity = block.arity
       min_arity = min_arity.abs - 1 if min_arity < 0
 
+      min_arity -=1 if base_object.is_a? Class # Instance-method procs take an extra param for receiving object
+
       if min_arity > arity_range.max
         raise ArgumentError.new("block requires #{min_arity} arguments while original_method require a maximum of #{arity_range.max}")
       end
@@ -281,6 +321,8 @@ module Spy
       @arity_range ||=
         if original_method
           min = max = 0
+          key_args = false
+          opt_keys = false
           original_method.parameters.each do |type,_|
             case type
             when :req
@@ -290,8 +332,17 @@ module Spy
               max += 1
             when :rest
               max = Float::INFINITY
+            when :keyreq
+              key_args = true
+            when :keyrest, :key
+              key_args = true
+              opt_keys = true
             end
           end
+          if key_args
+            max += 1
+            min += 1 unless opt_keys
+          end
           (min..max)
         end
     end
