@@ -38,6 +38,18 @@ class Pen
     "another"
   end
 
+  def opt_kwargs(required, opt: nil, opt2: nil)
+    [required, opt: opt, opt2: opt2]
+  end
+
+  def keyrest(**kwargs)
+    kwargs
+  end
+
+  def req_kwargs(req1:, req2:)
+    [req1, req2]
+  end
+
   protected
   def protected_method
   end
@@ -65,13 +77,11 @@ class Pen
   end
 end
 
-another = "meta_method"
-
-Pen.define_singleton_method(:meta_method) do
-  another
+Pen.define_singleton_method(:meta_class_method) do
+  "meta_class_method".freeze
 end
 
 Pen.send(:define_method, :meta_method) do
-  another
+  "meta_method".freeze
 end
 
